@@ -23,6 +23,8 @@
 #include <config.h>
 #endif
 
+#include <os/w32tiger.h>
+
 /*
  * Get VNC home directory ($HOME/.vnc or %APPDATA%/vnc/).
  * If HOME environment variable is set then it is used.
@@ -37,13 +39,4 @@
  */
 int getvnchomedir(char **dirp);
 
-/*
- * Check if the file exists
- *
- * Returns:
- * 0 - Success
- * -1 - Failure
- */
-int fileexists(char *file);
-
 #endif /* OS_OS_H */
