@@ -24,38 +24,32 @@
 #ifndef __RDR_ZLIBINSTREAM_H__
 #define __RDR_ZLIBINSTREAM_H__
 
-#include <rdr/InStream.h>
+#include <rdr/BufferedInStream.h>
 
 struct z_stream_s;
 
 namespace rdr {
 
-  class ZlibInStream : public InStream {
+  class ZlibInStream : public BufferedInStream {
 
   public:
-
-    ZlibInStream(size_t bufSize=0);
+    ZlibInStream();
     virtual ~ZlibInStream();
 
     void setUnderlying(InStream* is, size_t bytesIn);
     void flushUnderlying();
-    size_t pos();
     void reset();
 
   private:
-
     void init();
     void deinit();
 
-    size_t overrun(size_t itemSize, size_t nItems, bool wait);
-    bool decompress(bool wait);
+    virtual bool fillBuffer(size_t maxSize);
 
+  private:
     InStream* underlying;
-    size_t bufSize;
-    size_t offset;
     z_stream_s* zs;
     size_t bytesIn;
-    U8* start;
   };
 
 } // end of namespace rdr
