@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002-2012 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2015 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -33,7 +33,6 @@ import java.io.*;
 
 import java.util.Vector;
 
-@SuppressWarnings({"rawtypes","unchecked"})
 public class ChannelSftp extends ChannelSession{
 
   static private final int LOCAL_MAXIMUM_PACKET_SIZE=32*1024;
@@ -150,6 +149,11 @@ public class ChannelSftp extends ChannelSession{
   private java.util.Hashtable extensions=null;
   private InputStream io_in=null;
 
+  private boolean extension_posix_rename = false;
+  private boolean extension_statvfs = false;
+  // private boolean extension_fstatvfs = false;
+  private boolean extension_hardlink = false;
+
 /*
 10. Changes from previous protocol versions
   The SSH File Transfer Protocol has changed over time, before it's
@@ -178,7 +182,15 @@ public class ChannelSftp extends ChannelSession{
   private String fEncoding=UTF8;
   private boolean fEncoding_is_utf8=true;
 
-  private RequestQueue rq = new RequestQueue(10);
+  private RequestQueue rq = new RequestQueue(16);
+
+  /**
+   * Specify how many requests may be sent at any one time.
+   * Increasing this value may slightly improve file transfer speed but will
+   * increase memory usage.  The default is 16 requests.
+   *
+   * @param bulk_requests how many requests may be outstanding at any one time.
+   */
   public void setBulkRequests(int bulk_requests) throws JSchException {
     if(bulk_requests>0) 
       rq = new RequestQueue(bulk_requests);
@@ -186,11 +198,18 @@ public class ChannelSftp extends ChannelSession{
       throw new JSchException("setBulkRequests: "+ 
                               bulk_requests+" must be greater than 0.");
   }
+
+  /**
+   * This method will return the value how many requests may be
+   * sent at any one time.
+   *
+   * @return how many requests may be sent at any one time.
+   */
   public int getBulkRequests(){
     return rq.size();
   }
 
-  ChannelSftp(){
+  public ChannelSftp(){
     super();
     setLocalWindowSizeMax(LOCAL_WINDOW_SIZE_MAX);
     setLocalWindowSize(LOCAL_WINDOW_SIZE_MAX);
@@ -249,8 +268,8 @@ public class ChannelSftp extends ChannelSession{
       type=header.type;             // 2 -> SSH_FXP_VERSION
       server_version=header.rid;
       //System.err.println("SFTP protocol server-version="+server_version);
+      extensions=new java.util.Hashtable();
       if(length>0){
-        extensions=new java.util.Hashtable();
         // extension data
         fill(buf, length);
         byte[] extension_name=null;
@@ -265,6 +284,28 @@ public class ChannelSftp extends ChannelSession{
         }
       }
 
+      if(extensions.get("posix-rename@openssh.com")!=null &&
+         extensions.get("posix-rename@openssh.com").equals("1")){
+        extension_posix_rename = true;
+      } 
+
+      if(extensions.get("statvfs@openssh.com")!=null &&
+         extensions.get("statvfs@openssh.com").equals("2")){
+        extension_statvfs = true;
+      } 
+
+      /*
+      if(extensions.get("fstatvfs@openssh.com")!=null &&
+         extensions.get("fstatvfs@openssh.com").equals("2")){
+        extension_fstatvfs = true;
+      } 
+      */
+
+      if(extensions.get("hardlink@openssh.com")!=null &&
+         extensions.get("hardlink@openssh.com").equals("1")){
+        extension_hardlink = true;
+      } 
+
       lcwd=new File(".").getCanonicalPath();
     }
     catch(Exception e){
@@ -330,6 +371,17 @@ public class ChannelSftp extends ChannelSession{
 		  SftpProgressMonitor monitor) throws SftpException{
     put(src, dst, monitor, OVERWRITE);
   }
+
+  /**
+   * Sends data from <code>src</code> file to <code>dst</code> file.
+   * The <code>mode</code> should be <code>OVERWRITE</code>,
+   * <code>RESUME</code> or <code>APPEND</code>.
+   *
+   * @param src source file
+   * @param dst destination file
+   * @param monitor progress monitor
+   * @param mode how data should be added to dst
+   */
   public void put(String src, String dst, 
 		  SftpProgressMonitor monitor, int mode) throws SftpException{
 
@@ -446,6 +498,17 @@ public class ChannelSftp extends ChannelSession{
 		  SftpProgressMonitor monitor) throws SftpException{
     put(src, dst, monitor, OVERWRITE);
   }
+
+  /**
+   * Sends data from the input stream <code>src</code> to <code>dst</code> file.
+   * The <code>mode</code> should be <code>OVERWRITE</code>,
+   * <code>RESUME</code> or <code>APPEND</code>.
+   *
+   * @param src input stream
+   * @param dst destination file
+   * @param monitor progress monitor
+   * @param mode how data should be added to dst
+   */
   public void put(InputStream src, String dst, 
 		  SftpProgressMonitor monitor, int mode) throws SftpException{
     try{
@@ -468,10 +531,6 @@ public class ChannelSftp extends ChannelSession{
         dst=(String)(v.elementAt(0));
       }
 
-      if(isRemoteDir(dst)){
-        throw new SftpException(SSH_FX_FAILURE, dst+" is a directory");
-      }
-
       if(monitor!=null){
         monitor.init(SftpProgressMonitor.PUT, 
                      "-", dst,
@@ -481,7 +540,13 @@ public class ChannelSftp extends ChannelSession{
       _put(src, dst, monitor, mode);
     }
     catch(Exception e){
-      if(e instanceof SftpException) throw (SftpException)e;
+      if(e instanceof SftpException) {
+        if(((SftpException)e).id == SSH_FX_FAILURE &&
+           isRemoteDir(dst)) {
+          throw new SftpException(SSH_FX_FAILURE, dst+" is a directory");
+        }
+        throw (SftpException)e;
+      }
       if(e instanceof Throwable)
         throw new SftpException(SSH_FX_FAILURE, e.toString(), (Throwable)e);
       throw new SftpException(SSH_FX_FAILURE, e.toString());
@@ -583,7 +648,6 @@ public class ChannelSftp extends ChannelSession{
           if((seq-1)==startid ||
              ((seq-startid)-ackcount)>=bulk_requests){
             while(((seq-startid)-ackcount)>=bulk_requests){
-              if(this.rwsize>=foo) break;
               if(checkStatus(ackid, header)){
                 int _ackid = ackid[0];
                 if(startid>_ackid || _ackid>seq-1){
@@ -635,6 +699,18 @@ public class ChannelSftp extends ChannelSession{
   public OutputStream put(String dst, final SftpProgressMonitor monitor, final int mode) throws SftpException{
     return put(dst, monitor, mode, 0);
   }
+
+  /**
+   * Sends data from the output stream to <code>dst</code> file.
+   * The <code>mode</code> should be <code>OVERWRITE</code>,
+   * <code>RESUME</code> or <code>APPEND</code>.
+   *
+   * @param dst destination file
+   * @param monitor progress monitor
+   * @param mode how data should be added to dst
+   * @param offset data will be added at offset
+   * @return output stream, which accepts data to be transferred.
+   */
   public OutputStream put(String dst, final SftpProgressMonitor monitor, final int mode, long offset) throws SftpException{
     try{
       ((MyPipedInputStream)io_in).updateReadSide();
@@ -987,7 +1063,16 @@ public class ChannelSftp extends ChannelSession{
         length=header.length;
         type=header.type;
 
-        RequestQueue.Request rr = rq.get(header.rid);
+        RequestQueue.Request rr = null;
+        try{
+          rr = rq.get(header.rid);
+        }
+        catch(RequestQueue.OutOfOrderException e){
+          request_offset = e.offset;
+          skip(header.length);
+          rq.cancel(header, buf);
+          continue;
+        }
 
         if(type==SSH_FXP_STATUS){
           fill(buf, length);
@@ -1079,6 +1164,12 @@ public class ChannelSftp extends ChannelSession{
 
 
   private class RequestQueue {
+    class OutOfOrderException extends Exception {
+      long offset;
+      OutOfOrderException(long offset){
+        this.offset=offset;
+      }
+    }
     class Request {
       int id;
       long offset;
@@ -1109,15 +1200,27 @@ public class ChannelSftp extends ChannelSession{
       count++;
     }
 
-    Request get(int id){
+    Request get(int id) throws OutOfOrderException, SftpException {
       count -= 1;
-      int i=head;
+      int i = head;
       head++; 
       if(head==rrq.length) head=0;
-      if(rrq[i].id!=id){
-        System.err.println("The request is not in order.");
+      if(rrq[i].id != id){
+        long offset = getOffset();
+        boolean find = false;
+        for(int j = 0; j<rrq.length; j++){
+          if(rrq[j].id == id){
+            find = true;
+            rrq[j].id = 0;
+            break;
+          }
+        }
+        if(find)
+          throw new OutOfOrderException(offset);
+        throw new SftpException(SSH_FX_FAILURE, 
+                                "RequestQueue: unknown request id "+id);
       }
-      rrq[i].id=0;
+      rrq[i].id = 0;
       return rrq[i];
     }
 
@@ -1134,10 +1237,29 @@ public class ChannelSftp extends ChannelSession{
       for(int i=0; i<_count; i++){
         header=header(buf, header);
         int length=header.length;
-        get(header.rid);
+        for(int j=0; j<rrq.length; j++){
+          if(rrq[j].id == header.rid){
+            rrq[j].id=0;
+            break;
+          }
+        }
         skip(length); 
       }
+      init();
     }
+
+    long getOffset(){
+      long result = Long.MAX_VALUE;
+
+      for(int i=0; i<rrq.length; i++){
+        if(rrq[i].id == 0)
+          continue;
+        if(result>rrq[i].offset)
+          result=rrq[i].offset;
+      }
+
+      return result;
+    } 
   }
 
   public InputStream get(String src) throws SftpException{
@@ -1193,6 +1315,8 @@ public class ChannelSftp extends ChannelSession{
 
       final byte[] handle=buf.getString();         // handle
 
+      rq.init();
+
       java.io.InputStream in=new java.io.InputStream(){
            long offset=skip;
            boolean closed=false;
@@ -1200,6 +1324,8 @@ public class ChannelSftp extends ChannelSession{
            byte[] _data=new byte[1];
            byte[] rest_byte=new byte[1024];
            Header header=new Header();
+           int request_max=1;
+           long request_offset=offset;
 
            public int read() throws java.io.IOException{
              if(closed)return -1;
@@ -1248,14 +1374,38 @@ public class ChannelSftp extends ChannelSession{
                len=1024; 
              }
 
-             try{sendREAD(handle, offset, len);}
-             catch(Exception e){ throw new IOException("error"); }
+             if(rq.count()==0) {
+               int request_len = buf.buffer.length-13;
+               if(server_version==0){ request_len=1024; }
+
+               while(rq.count() < request_max){
+                 try{
+                   sendREAD(handle, request_offset, request_len, rq);
+                 }
+                 catch(Exception e){ throw new IOException("error"); }
+                 request_offset += request_len;
+               }
+             }
 
              header=header(buf, header);
              rest_length=header.length;
              int type=header.type;
              int id=header.rid;
 
+             RequestQueue.Request rr = null;
+             try{
+               rr = rq.get(header.rid);
+             }
+             catch(RequestQueue.OutOfOrderException e){
+               request_offset = e.offset;
+               skip(header.length);
+               rq.cancel(header, buf);
+               return 0;
+             }
+             catch(SftpException e){
+               throw new IOException("error: "+e.toString());
+             }
+
              if(type!=SSH_FXP_STATUS && type!=SSH_FXP_DATA){ 
                throw new IOException("error");
              }
@@ -1270,6 +1420,7 @@ public class ChannelSftp extends ChannelSession{
                //throwStatusError(buf, i);
                throw new IOException("error");
              }
+
              buf.rewind();
              fill(buf.buffer, 0, 4);
              int length_of_data = buf.getInt(); rest_length-=4;
@@ -1319,6 +1470,21 @@ public class ChannelSftp extends ChannelSession{
                  io_in.skip(optional_data);
                }
 
+               if(length_of_data<rr.length){  //
+                 rq.cancel(header, buf);
+                 try {
+                   sendREAD(handle,
+                            rr.offset+length_of_data,
+                            (int)(rr.length-length_of_data), rq);
+                 }
+                 catch(Exception e){ throw new IOException("error"); }
+                 request_offset=rr.offset+rr.length;
+               }
+
+               if(request_max < rq.size()){
+                 request_max++;
+               }
+
                if(monitor!=null){
                  if(!monitor.count(i)){
                    close();
@@ -1334,6 +1500,7 @@ public class ChannelSftp extends ChannelSession{
              if(closed)return;
              closed=true;
              if(monitor!=null)monitor.end();
+             rq.cancel(header, buf);
              try{_sendCLOSE(handle, header);}
              catch(Exception e){throw new IOException("error");}
            }
@@ -1349,6 +1516,28 @@ public class ChannelSftp extends ChannelSession{
    }
 
    public java.util.Vector ls(String path) throws SftpException{
+     final java.util.Vector v = new Vector();
+     LsEntrySelector selector = new LsEntrySelector(){
+       public int select(LsEntry entry){
+         v.addElement(entry);
+         return CONTINUE;
+       }
+     };
+     ls(path, selector);
+     return v;
+   }
+
+  /**
+   * List files specified by the remote <code>path</code>.
+   * Each files and directories will be passed to
+   * <code>LsEntrySelector#select(LsEntry)</code> method, and if that method
+   * returns <code>LsEntrySelector#BREAK</code>, the operation will be
+   * canceled immediately.
+   *
+   * @see ChannelSftp.LsEntrySelector
+   * @since 0.1.47
+   */
+   public void ls(String path, LsEntrySelector selector) throws SftpException{
      //System.out.println("ls: "+path);
      try{
        ((MyPipedInputStream)io_in).updateReadSide();
@@ -1417,9 +1606,11 @@ public class ChannelSftp extends ChannelSession{
          throwStatusError(buf, i);
        }
 
+       int cancel = LsEntrySelector.CONTINUE;
        byte[] handle=buf.getString();         // handle
 
-       while(true){
+       while(cancel==LsEntrySelector.CONTINUE){
+
          sendREADDIR(handle);
 
          header=header(buf, header);
@@ -1461,6 +1652,11 @@ public class ChannelSftp extends ChannelSession{
            }
            SftpATTRS attrs=SftpATTRS.getATTR(buf);
 
+           if(cancel==LsEntrySelector.BREAK){
+             count--; 
+             continue;
+           }
+
            boolean find=false;
            String f=null;
            if(pattern==null){
@@ -1491,7 +1687,8 @@ public class ChannelSftp extends ChannelSession{
              else{
                l=Util.byte2str(longname, fEncoding);
              }
-             v.addElement(new LsEntry(f, l, attrs));
+
+             cancel = selector.select(new LsEntry(f, l, attrs));
            }
 
            count--; 
@@ -1516,7 +1713,6 @@ public class ChannelSftp extends ChannelSession{
        }
        */
 
-       return v;
      }
      catch(Exception e){
        if(e instanceof SftpException) throw (SftpException)e;
@@ -1525,6 +1721,7 @@ public class ChannelSftp extends ChannelSession{
        throw new SftpException(SSH_FX_FAILURE, "");
      }
    }
+
    public String readlink(String path) throws SftpException{
      try{
        if(server_version<3){
@@ -1574,6 +1771,7 @@ public class ChannelSftp extends ChannelSession{
      }
      return null;
    }
+
    public void symlink(String oldpath, String newpath) throws SftpException{
      if(server_version<3){
        throw new SftpException(SSH_FX_OP_UNSUPPORTED, 
@@ -1583,10 +1781,17 @@ public class ChannelSftp extends ChannelSession{
      try{
        ((MyPipedInputStream)io_in).updateReadSide();
 
-       oldpath=remoteAbsolutePath(oldpath);
+       String _oldpath=remoteAbsolutePath(oldpath);
        newpath=remoteAbsolutePath(newpath);
 
-       oldpath=isUnique(oldpath);
+       _oldpath=isUnique(_oldpath);
+       if(oldpath.charAt(0)!='/'){ // relative path
+         String cwd=getCwd();
+         oldpath=_oldpath.substring(cwd.length()+(cwd.endsWith("/")?0:1));
+       }
+       else {
+         oldpath=_oldpath;
+       }
 
        if(isPattern(newpath)){
          throw new SftpException(SSH_FX_FAILURE, newpath);
@@ -1619,6 +1824,58 @@ public class ChannelSftp extends ChannelSession{
      }
    }
 
+   public void hardlink(String oldpath, String newpath) throws SftpException{
+     if(!extension_hardlink){
+       throw new SftpException(SSH_FX_OP_UNSUPPORTED, 
+                               "hardlink@openssh.com is not supported");
+     }
+
+     try{
+       ((MyPipedInputStream)io_in).updateReadSide();
+
+       String _oldpath=remoteAbsolutePath(oldpath);
+       newpath=remoteAbsolutePath(newpath);
+
+       _oldpath=isUnique(_oldpath);
+       if(oldpath.charAt(0)!='/'){ // relative path
+         String cwd=getCwd();
+         oldpath=_oldpath.substring(cwd.length()+(cwd.endsWith("/")?0:1));
+       }
+       else {
+         oldpath=_oldpath;
+       }
+
+       if(isPattern(newpath)){
+         throw new SftpException(SSH_FX_FAILURE, newpath);
+       }
+       newpath=Util.unquote(newpath);
+
+       sendHARDLINK(Util.str2byte(oldpath, fEncoding),
+                   Util.str2byte(newpath, fEncoding));
+
+       Header header=new Header();
+       header=header(buf, header);
+       int length=header.length;
+       int type=header.type;
+
+       fill(buf, length);
+
+       if(type!=SSH_FXP_STATUS){
+         throw new SftpException(SSH_FX_FAILURE, "");
+       }
+
+       int i=buf.getInt();
+       if(i==SSH_FX_OK) return;
+       throwStatusError(buf, i);
+     }
+     catch(Exception e){
+       if(e instanceof SftpException) throw (SftpException)e;
+       if(e instanceof Throwable)
+         throw new SftpException(SSH_FX_FAILURE, "", (Throwable)e);
+       throw new SftpException(SSH_FX_FAILURE, "");
+     }
+   }
+
    public void rename(String oldpath, String newpath) throws SftpException{
      if(server_version<2){
        throw new SftpException(SSH_FX_OP_UNSUPPORTED, 
@@ -1958,6 +2215,66 @@ public class ChannelSftp extends ChannelSession{
     return _stat(Util.str2byte(path, fEncoding));
   }
 
+  public SftpStatVFS statVFS(String path) throws SftpException{
+    try{
+      ((MyPipedInputStream)io_in).updateReadSide();
+
+      path=remoteAbsolutePath(path);
+      path=isUnique(path);
+
+      return _statVFS(path);
+    }
+    catch(Exception e){
+      if(e instanceof SftpException) throw (SftpException)e;
+      if(e instanceof Throwable)
+        throw new SftpException(SSH_FX_FAILURE, "", (Throwable)e);
+      throw new SftpException(SSH_FX_FAILURE, "");
+    }
+    //return null;
+  }
+
+  private SftpStatVFS _statVFS(byte[] path) throws SftpException{
+    if(!extension_statvfs){
+      throw new SftpException(SSH_FX_OP_UNSUPPORTED, 
+                              "statvfs@openssh.com is not supported");
+    }
+
+    try{
+
+      sendSTATVFS(path);
+
+      Header header=new Header();
+      header=header(buf, header);
+      int length=header.length;
+      int type=header.type;
+
+      fill(buf, length);
+
+      if(type != (SSH_FXP_EXTENDED_REPLY&0xff)){
+        if(type==SSH_FXP_STATUS){
+          int i=buf.getInt();
+          throwStatusError(buf, i);
+        }
+        throw new SftpException(SSH_FX_FAILURE, "");
+      }
+      else {
+        SftpStatVFS stat = SftpStatVFS.getStatVFS(buf);
+        return stat;
+      }
+    }
+    catch(Exception e){
+      if(e instanceof SftpException) throw (SftpException)e;
+      if(e instanceof Throwable)
+        throw new SftpException(SSH_FX_FAILURE, "", (Throwable)e);
+      throw new SftpException(SSH_FX_FAILURE, "");
+    }
+    //return null;
+  }
+
+  private SftpStatVFS _statVFS(String path) throws SftpException{
+    return _statVFS(Util.str2byte(path, fEncoding));
+  }
+
   public SftpATTRS lstat(String path) throws SftpException{
     try{
       ((MyPipedInputStream)io_in).updateReadSide();
@@ -2161,6 +2478,14 @@ public class ChannelSftp extends ChannelSession{
   private void sendSTAT(byte[] path) throws Exception{
     sendPacketPath(SSH_FXP_STAT, path);
   }
+  private void sendSTATVFS(byte[] path) throws Exception{
+    sendPacketPath((byte)0, path, "statvfs@openssh.com");
+  }
+  /*
+  private void sendFSTATVFS(byte[] handle) throws Exception{
+    sendPacketPath((byte)0, handle, "fstatvfs@openssh.com");
+  }
+  */
   private void sendLSTAT(byte[] path) throws Exception{
     sendPacketPath(SSH_FXP_LSTAT, path);
   }
@@ -2193,6 +2518,9 @@ public class ChannelSftp extends ChannelSession{
   private void sendSYMLINK(byte[] p1, byte[] p2) throws Exception{
     sendPacketPath(SSH_FXP_SYMLINK, p1, p2);
   }
+  private void sendHARDLINK(byte[] p1, byte[] p2) throws Exception{
+    sendPacketPath((byte)0, p1, p2, "hardlink@openssh.com");
+  }
   private void sendREADLINK(byte[] path) throws Exception{
     sendPacketPath(SSH_FXP_READLINK, path);
   }
@@ -2203,7 +2531,8 @@ public class ChannelSftp extends ChannelSession{
     sendPacketPath(SSH_FXP_READDIR, path);
   }
   private void sendRENAME(byte[] p1, byte[] p2) throws Exception{
-    sendPacketPath(SSH_FXP_RENAME, p1, p2);
+    sendPacketPath(SSH_FXP_RENAME, p1, p2,
+                   extension_posix_rename ? "posix-rename@openssh.com" : null);
   }
   private void sendCLOSE(byte[] path) throws Exception{
     sendPacketPath(SSH_FXP_CLOSE, path);
@@ -2227,19 +2556,44 @@ public class ChannelSftp extends ChannelSession{
     getSession().write(packet, this, 17+path.length+4);
   }
   private void sendPacketPath(byte fxp, byte[] path) throws Exception{
-    packet.reset();
-    putHEAD(fxp, 9+path.length);
-    buf.putInt(seq++);
-    buf.putString(path);             // path
-    getSession().write(packet, this, 9+path.length+4);
+    sendPacketPath(fxp, path, (String)null);
   }
-  private void sendPacketPath(byte fxp, byte[] p1, byte[] p2) throws Exception{
+  private void sendPacketPath(byte fxp, byte[] path, String extension) throws Exception{
     packet.reset();
-    putHEAD(fxp, 13+p1.length+p2.length);
-    buf.putInt(seq++);
+    int len = 9+path.length;
+    if(extension == null) {
+      putHEAD(fxp, len);
+      buf.putInt(seq++);
+    }
+    else {
+      len+=(4+extension.length());
+      putHEAD(SSH_FXP_EXTENDED, len);
+      buf.putInt(seq++);
+      buf.putString(Util.str2byte(extension));
+    }
+    buf.putString(path);             // path
+    getSession().write(packet, this, len+4);
+  }
+
+  private void sendPacketPath(byte fxp, byte[] p1, byte[] p2) throws Exception{
+    sendPacketPath(fxp, p1, p2, null);
+  }
+  private void sendPacketPath(byte fxp, byte[] p1, byte[] p2, String extension) throws Exception{
+    packet.reset();
+    int len = 13+p1.length+p2.length;
+    if(extension==null){
+      putHEAD(fxp, len);
+      buf.putInt(seq++);
+    }
+    else {
+      len+=(4+extension.length());
+      putHEAD(SSH_FXP_EXTENDED, len);
+      buf.putInt(seq++);
+      buf.putString(Util.str2byte(extension));
+    }
     buf.putString(p1);
     buf.putString(p2);
-    getSession().write(packet, this, 13+p1.length+p2.length+4);
+    getSession().write(packet, this, len+4);
   }
 
   private int sendWRITE(byte[] handle, long offset, 
@@ -2649,4 +3003,26 @@ public class ChannelSftp extends ChannelSession{
       throw new ClassCastException("a decendent of LsEntry must be given.");
     }
   }
+
+  /**
+   * This interface will be passed as an argument for <code>ls</code> method.
+   *
+   * @see ChannelSftp.LsEntry
+   * @see #ls(String, ChannelSftp.LsEntrySelector)
+   * @since 0.1.47
+   */
+  public interface LsEntrySelector {
+    public final int CONTINUE = 0;
+    public final int BREAK = 1;
+
+    /**
+     * <p> The <code>select</code> method will be invoked in <code>ls</code>
+     * method for each file entry. If this method returns BREAK,
+     * <code>ls</code> will be canceled.
+     * 
+     * @param entry one of entry from ls
+     * @return if BREAK is returned, the 'ls' operation will be canceled.
+     */
+    public int select(LsEntry entry);
+  }
 }
