@@ -22,11 +22,6 @@
 
 #include <x0vncserver/TimeMillis.h>
 
-// XXX Lynx/OS 2.3: get proto for gettimeofday()
-#ifdef Lynx
-#include <sys/proto.h>
-#endif
-
 TimeMillis::TimeMillis()
 {
   update();
