@@ -27,7 +27,6 @@
 #include "dixstruct.h"
 #include "extnsionst.h"
 #include "scrnintstr.h"
-#include "selection.h"
 
 #define _VNCEXT_SERVER_
 #define _VNCEXT_PROTO_
@@ -44,17 +43,11 @@ static void vncResetProc(ExtensionEntry* extEntry);
 
 static void vncClientStateChange(CallbackListPtr*, void *, void *);
 
-static void vncSelectionCallback(CallbackListPtr *callbacks,
-                                 void * data, void * args);
-
 static int vncErrorBase = 0;
 static int vncEventBase = 0;
 
 int vncNoClipboard = 0;
 
-static char* clientCutText = NULL;
-static int clientCutTextLen = 0;
-
 static struct VncInputSelect* vncInputSelectHead = NULL;
 
 struct VncInputSelect {
@@ -64,7 +57,7 @@ struct VncInputSelect {
   struct VncInputSelect* next;
 };
 
-int vncAddExtension(void)
+void vncAddExtension(void)
 {
   ExtensionEntry* extEntry;
 
@@ -72,8 +65,7 @@ int vncAddExtension(void)
                           ProcVncExtDispatch, SProcVncExtDispatch, vncResetProc,
                           StandardMinorOpcode);
   if (!extEntry) {
-    ErrorF("vncAddExtension: AddExtension failed\n");
-    return -1;
+    FatalError("vncAddExtension: AddExtension failed\n");
   }
 
   vncErrorBase = extEntry->errorBase;
@@ -82,12 +74,6 @@ int vncAddExtension(void)
   if (!AddCallback(&ClientStateCallback, vncClientStateChange, 0)) {
     FatalError("Add ClientStateCallback failed\n");
   }
-
-  if (!AddCallback(&SelectionCallback, vncSelectionCallback, 0)) {
-    FatalError("Add SelectionCallback failed\n");
-  }
-
-  return 0;
 }
 
 int vncNotifyQueryConnect(void)
@@ -103,14 +89,8 @@ int vncNotifyQueryConnect(void)
       ev.sequenceNumber = cur->client->sequence;
       ev.window = cur->window;
       if (cur->client->swapped) {
-#if XORG < 112
-        int n;
-        swaps(&ev.sequenceNumber, n);
-        swapl(&ev.window, n);
-#else
         swaps(&ev.sequenceNumber);
         swapl(&ev.window);
-#endif
       }
       WriteToClient(cur->client, sizeof(xVncExtQueryConnectNotifyEvent),
                     (char *)&ev);
@@ -121,47 +101,6 @@ int vncNotifyQueryConnect(void)
   return count;
 }
 
-void vncClientCutText(const char* str, int len)
-{
-  xVncExtClientCutTextNotifyEvent ev;
-
-  if (clientCutText != NULL)
-    free(clientCutText);
-  clientCutTextLen = 0;
-
-  clientCutText = malloc(len);
-  if (clientCutText == NULL) {
-    ErrorF("Could not allocate clipboard buffer\n");
-    return;
-  }
-
-  memcpy(clientCutText, str, len);
-  clientCutTextLen = len;
-
-  ev.type = vncEventBase + VncExtClientCutTextNotify;
-  for (struct VncInputSelect* cur = vncInputSelectHead; cur; cur = cur->next) {
-    if (cur->mask & VncExtClientCutTextMask) {
-      ev.sequenceNumber = cur->client->sequence;
-      ev.window = cur->window;
-      ev.time = GetTimeInMillis();
-      if (cur->client->swapped) {
-#if XORG < 112
-        int n;
-        swaps(&ev.sequenceNumber, n);
-        swapl(&ev.window, n);
-        swapl(&ev.time, n);
-#else
-        swaps(&ev.sequenceNumber);
-        swapl(&ev.window);
-        swapl(&ev.time);
-#endif
-      }
-      WriteToClient(cur->client, sizeof(xVncExtClientCutTextNotifyEvent),
-                    (char *)&ev);
-    }
-  }
-}
-
 static int ProcVncExtSetParam(ClientPtr client)
 {
   char *param;
@@ -202,14 +141,8 @@ deny:
   free(param);
 
   if (client->swapped) {
-#if XORG < 112
-    int n;
-    swaps(&rep.sequenceNumber, n);
-    swapl(&rep.length, n);
-#else
     swaps(&rep.sequenceNumber);
     swapl(&rep.length);
-#endif
   }
   WriteToClient(client, sizeof(xVncExtSetParamReply), (char *)&rep);
   return (client->noClientException);
@@ -218,12 +151,7 @@ deny:
 static int SProcVncExtSetParam(ClientPtr client)
 {
   REQUEST(xVncExtSetParamReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
   swaps(&stuff->length);
-#endif
   REQUEST_AT_LEAST_SIZE(xVncExtSetParamReq);
   return ProcVncExtSetParam(client);
 }
@@ -257,16 +185,9 @@ static int ProcVncExtGetParam(ClientPtr client)
   rep.length = (len + 3) >> 2;
   rep.valueLen = len;
   if (client->swapped) {
-#if XORG < 112
-    int n;
-    swaps(&rep.sequenceNumber, n);
-    swapl(&rep.length, n);
-    swaps(&rep.valueLen, n);
-#else
     swaps(&rep.sequenceNumber);
     swapl(&rep.length);
     swaps(&rep.valueLen);
-#endif
   }
   WriteToClient(client, sizeof(xVncExtGetParamReply), (char *)&rep);
   if (value)
@@ -278,12 +199,7 @@ static int ProcVncExtGetParam(ClientPtr client)
 static int SProcVncExtGetParam(ClientPtr client)
 {
   REQUEST(xVncExtGetParamReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
   swaps(&stuff->length);
-#endif
   REQUEST_AT_LEAST_SIZE(xVncExtGetParamReq);
   return ProcVncExtGetParam(client);
 }
@@ -317,16 +233,9 @@ static int ProcVncExtGetParamDesc(ClientPtr client)
   rep.length = (len + 3) >> 2;
   rep.descLen = len;
   if (client->swapped) {
-#if XORG < 112
-    int n;
-    swaps(&rep.sequenceNumber, n);
-    swapl(&rep.length, n);
-    swaps(&rep.descLen, n);
-#else
     swaps(&rep.sequenceNumber);
     swapl(&rep.length);
     swaps(&rep.descLen);
-#endif
   }
   WriteToClient(client, sizeof(xVncExtGetParamDescReply), (char *)&rep);
   if (desc)
@@ -337,12 +246,7 @@ static int ProcVncExtGetParamDesc(ClientPtr client)
 static int SProcVncExtGetParamDesc(ClientPtr client)
 {
   REQUEST(xVncExtGetParamDescReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
   swaps(&stuff->length);
-#endif
   REQUEST_AT_LEAST_SIZE(xVncExtGetParamDescReq);
   return ProcVncExtGetParamDesc(client);
 }
@@ -367,16 +271,9 @@ static int ProcVncExtListParams(ClientPtr client)
   rep.length = (len + 3) >> 2;
   rep.nParams = vncGetParamCount();
   if (client->swapped) {
-#if XORG < 112
-    int n;
-    swaps(&rep.sequenceNumber, n);
-    swapl(&rep.length, n);
-    swaps(&rep.nParams, n);
-#else
     swaps(&rep.sequenceNumber);
     swapl(&rep.length);
     swaps(&rep.nParams);
-#endif
   }
   WriteToClient(client, sizeof(xVncExtListParamsReply), (char *)&rep);
   WriteToClient(client, len, (char*)params);
@@ -387,83 +284,11 @@ static int ProcVncExtListParams(ClientPtr client)
 static int SProcVncExtListParams(ClientPtr client)
 {
   REQUEST(xVncExtListParamsReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
   swaps(&stuff->length);
-#endif
   REQUEST_SIZE_MATCH(xVncExtListParamsReq);
   return ProcVncExtListParams(client);
 }
 
-static int ProcVncExtSetServerCutText(ClientPtr client)
-{
-  REQUEST(xVncExtSetServerCutTextReq);
-  REQUEST_FIXED_SIZE(xVncExtSetServerCutTextReq, stuff->textLen);
-  vncServerCutText((const char*)&stuff[1], stuff->textLen);
-  return (client->noClientException);
-}
-
-static int SProcVncExtSetServerCutText(ClientPtr client)
-{
-  REQUEST(xVncExtSetServerCutTextReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
-  swaps(&stuff->length);
-#endif
-  REQUEST_AT_LEAST_SIZE(xVncExtSetServerCutTextReq);
-#if XORG < 112
-  swapl(&stuff->textLen, n);
-#else
-  swapl(&stuff->textLen);
-#endif
-  return ProcVncExtSetServerCutText(client);
-}
-
-static int ProcVncExtGetClientCutText(ClientPtr client)
-{
-  xVncExtGetClientCutTextReply rep;
-
-  REQUEST_SIZE_MATCH(xVncExtGetClientCutTextReq);
-
-  rep.type = X_Reply;
-  rep.length = (clientCutTextLen + 3) >> 2;
-  rep.sequenceNumber = client->sequence;
-  rep.textLen = clientCutTextLen;
-  if (client->swapped) {
-#if XORG < 112
-    int n;
-    swaps(&rep.sequenceNumber, n);
-    swapl(&rep.length, n);
-    swapl(&rep.textLen, n);
-#else
-    swaps(&rep.sequenceNumber);
-    swapl(&rep.length);
-    swapl(&rep.textLen);
-#endif
-  }
-  WriteToClient(client, sizeof(xVncExtGetClientCutTextReply), (char *)&rep);
-  if (clientCutText)
-    WriteToClient(client, clientCutTextLen, clientCutText);
-  return (client->noClientException);
-}
-
-static int SProcVncExtGetClientCutText(ClientPtr client)
-{
-  REQUEST(xVncExtGetClientCutTextReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
-  swaps(&stuff->length);
-#endif
-  REQUEST_SIZE_MATCH(xVncExtGetClientCutTextReq);
-  return ProcVncExtGetClientCutText(client);
-}
-
 static int ProcVncExtSelectInput(ClientPtr client)
 {
   struct VncInputSelect** nextPtr;
@@ -501,20 +326,10 @@ static int ProcVncExtSelectInput(ClientPtr client)
 static int SProcVncExtSelectInput(ClientPtr client)
 {
   REQUEST(xVncExtSelectInputReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
   swaps(&stuff->length);
-#endif
   REQUEST_SIZE_MATCH(xVncExtSelectInputReq);
-#if XORG < 112
-  swapl(&stuff->window, n);
-  swapl(&stuff->mask, n);
-#else
   swapl(&stuff->window);
   swapl(&stuff->mask);
-#endif
   return ProcVncExtSelectInput(client);
 }
 
@@ -540,14 +355,8 @@ static int ProcVncExtConnect(ClientPtr client)
   rep.length = 0;
   rep.sequenceNumber = client->sequence;
   if (client->swapped) {
-#if XORG < 112
-    int n;
-    swaps(&rep.sequenceNumber, n);
-    swapl(&rep.length, n);
-#else
     swaps(&rep.sequenceNumber);
     swapl(&rep.length);
-#endif
   }
   WriteToClient(client, sizeof(xVncExtConnectReply), (char *)&rep);
 
@@ -559,12 +368,7 @@ static int ProcVncExtConnect(ClientPtr client)
 static int SProcVncExtConnect(ClientPtr client)
 {
   REQUEST(xVncExtConnectReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
   swaps(&stuff->length);
-#endif
   REQUEST_AT_LEAST_SIZE(xVncExtConnectReq);
   return ProcVncExtConnect(client);
 }
@@ -590,22 +394,12 @@ static int ProcVncExtGetQueryConnect(ClientPtr client)
   rep.opaqueId = (CARD32)(long)opaqueId;
   rep.length = ((rep.userLen + 3) >> 2) + ((rep.addrLen + 3) >> 2);
   if (client->swapped) {
-#if XORG < 112
-    int n;
-    swaps(&rep.sequenceNumber, n);
-    swapl(&rep.addrLen, n);
-    swapl(&rep.userLen, n);
-    swapl(&rep.timeout, n);
-    swapl(&rep.opaqueId, n);
-    swapl(&rep.length, n);
-#else
     swaps(&rep.sequenceNumber);
     swapl(&rep.addrLen);
     swapl(&rep.userLen);
     swapl(&rep.timeout);
     swapl(&rep.opaqueId);
     swapl(&rep.length);
-#endif
   }
   WriteToClient(client, sizeof(xVncExtGetQueryConnectReply), (char *)&rep);
   if (qcTimeout)
@@ -618,12 +412,7 @@ static int ProcVncExtGetQueryConnect(ClientPtr client)
 static int SProcVncExtGetQueryConnect(ClientPtr client)
 {
   REQUEST(xVncExtGetQueryConnectReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-#else
   swaps(&stuff->length);
-#endif
   REQUEST_SIZE_MATCH(xVncExtGetQueryConnectReq);
   return ProcVncExtGetQueryConnect(client);
 }
@@ -642,14 +431,8 @@ static int ProcVncExtApproveConnect(ClientPtr client)
 static int SProcVncExtApproveConnect(ClientPtr client)
 {
   REQUEST(xVncExtApproveConnectReq);
-#if XORG < 112
-  register char n;
-  swaps(&stuff->length, n);
-  swapl(&stuff->opaqueId, n);
-#else
   swaps(&stuff->length);
   swapl(&stuff->opaqueId);
-#endif
   REQUEST_SIZE_MATCH(xVncExtApproveConnectReq);
   return ProcVncExtApproveConnect(client);
 }
@@ -667,10 +450,6 @@ static int ProcVncExtDispatch(ClientPtr client)
     return ProcVncExtGetParamDesc(client);
   case X_VncExtListParams:
     return ProcVncExtListParams(client);
-  case X_VncExtSetServerCutText:
-    return ProcVncExtSetServerCutText(client);
-  case X_VncExtGetClientCutText:
-    return ProcVncExtGetClientCutText(client);
   case X_VncExtSelectInput:
     return ProcVncExtSelectInput(client);
   case X_VncExtConnect:
@@ -696,10 +475,6 @@ static int SProcVncExtDispatch(ClientPtr client)
     return SProcVncExtGetParamDesc(client);
   case X_VncExtListParams:
     return SProcVncExtListParams(client);
-  case X_VncExtSetServerCutText:
-    return SProcVncExtSetServerCutText(client);
-  case X_VncExtGetClientCutText:
-    return SProcVncExtGetClientCutText(client);
   case X_VncExtSelectInput:
     return SProcVncExtSelectInput(client);
   case X_VncExtConnect:
@@ -715,6 +490,7 @@ static int SProcVncExtDispatch(ClientPtr client)
 
 static void vncResetProc(ExtensionEntry* extEntry)
 {
+  vncExtensionClose();
 }
 
 static void vncClientStateChange(CallbackListPtr * l, void * d, void * p)
@@ -732,39 +508,3 @@ static void vncClientStateChange(CallbackListPtr * l, void * d, void * p)
     }
   }
 }
-
-static void SendSelectionChangeEvent(Atom selection)
-{
-  xVncExtSelectionChangeNotifyEvent ev;
-  ev.type = vncEventBase + VncExtSelectionChangeNotify;
-  for (struct VncInputSelect* cur = vncInputSelectHead; cur; cur = cur->next) {
-    if (cur->mask & VncExtSelectionChangeMask) {
-      ev.sequenceNumber = cur->client->sequence;
-      ev.window = cur->window;
-      ev.selection = selection;
-      if (cur->client->swapped) {
-#if XORG < 112
-        int n;
-        swaps(&ev.sequenceNumber, n);
-        swapl(&ev.window, n);
-        swapl(&ev.selection, n);
-#else
-        swaps(&ev.sequenceNumber);
-        swapl(&ev.window);
-        swapl(&ev.selection);
-#endif
-      }
-      WriteToClient(cur->client, sizeof(xVncExtSelectionChangeNotifyEvent),
-                    (char *)&ev);
-    }
-  }
-}
-
-static void vncSelectionCallback(CallbackListPtr *callbacks, void * data, void * args)
-{
-  SelectionInfoRec *info = (SelectionInfoRec *) args;
-  Selection *selection = info->selection;
-
-  SendSelectionChangeEvent(selection->selection);
-}
-
