@@ -62,21 +62,27 @@ public:
   // Resize the current framebuffer, but retain the contents
   void resizeFramebuffer(int new_w, int new_h);
 
-  // Incoming clipboard from server
-  void serverCutText(const char* str, rdr::U32 len);
-
   // New image for the locally rendered cursor
   void setCursor(int width, int height, const rfb::Point& hotspot,
                  const rdr::U8* data);
 
+  // Server-provided cursor position
+  void setCursorPos(const rfb::Point& pos);
+
   // Change client LED state
   void setLEDState(unsigned int state);
 
-  // Fl_Window callback methods
-  void draw();
-  void resize(int x, int y, int w, int h);
+  // Clipboard events
+  void handleClipboardRequest();
+  void handleClipboardAnnounce(bool available);
+  void handleClipboardData(const char* data);
 
-  int handle(int event);
+  // Fl_Window callback methods
+  virtual void show();
+  virtual void draw();
+  virtual void resize(int x, int y, int w, int h);
+
+  virtual int handle(int event);
 
   void fullscreen_on();
 
@@ -86,8 +92,12 @@ private:
   void setOverlay(const char *text, ...) __printf_attr(2, 3);
   static void updateOverlay(void *data);
 
-  static int fltkHandle(int event, Fl_Window *win);
+  static int fltkDispatch(int event, Fl_Window *win);
+  static int fltkHandle(int event);
 
+  bool hasFocus();
+
+  void maybeGrabKeyboard();
   void grabKeyboard();
   void ungrabKeyboard();
   void grabPointer();
@@ -99,6 +109,7 @@ private:
 
   void handleDesktopSize();
   static void handleResizeTimeout(void *data);
+  static void reconfigureFullscreen(void *data);
   void remoteResize(int width, int height);
 
   void repositionWidgets();
