@@ -26,6 +26,8 @@
 
 #include <FL/Fl_Widget.H>
 
+#include "EmulateMB.h"
+
 class Fl_Menu_Button;
 class Fl_RGB_Image;
 
@@ -33,7 +35,7 @@ class CConn;
 class PlatformPixelBuffer;
 class Surface;
 
-class Viewport : public Fl_Widget {
+class Viewport : public Fl_Widget, public EmulateMB {
 public:
 
   Viewport(int w, int h, const rfb::PixelFormat& serverPF, CConn* cc_);
@@ -67,6 +69,9 @@ public:
 
   int handle(int event);
 
+protected:
+  virtual void sendPointerEvent(const rfb::Point& pos, int buttonMask);
+
 private:
   bool hasFocus();
 
@@ -86,6 +91,7 @@ private:
 
 #ifdef WIN32
   static void handleAltGrTimeout(void *data);
+  void resolveAltGrDetection(bool isAltGrSequence);
 #endif
 
   void pushLEDState();
